# -*- coding: utf-8 -*-

#1a) Due configurazioni della stessa proteina sono caratterizzate da diverse 
# sequenze delle coppie di angoli (phi, psi). Si scriva una funzione 
# diff_ang(pdb1, pdb2) che, presi in ingresso due file PDB corrispondenti a 
# due configurazioni proteiche a catena singola, restutuisca la lista delle 
# tuple che contengono le due differenze angolari tra le due configurazioni, 
# ovvero (phi2-phi1, psi2-psi1). Si assuma che gli angoli siano misurati in 
# radianti. Notare che phi non è definito per il primo residuo, mentre psi non 
# lo è per l'ultimo.
# Tale funzione dovrà lanciare un'eccezione del tipo ValueError se le 
# configurazioni si riferiscono a catene di diversa lunghezza, o se almeno uno 
# dei file PDB contiene più di una catena (o più di un modello). 
# Suggerimento: trattare le catene proteiche caricate da file PDB come 
# istanze della classe "Polypeptide" di Biopython; a riguardo, leggere le note
# allegate.

from Bio.PDB.PDBParser import PDBParser
from Bio.PDB.Polypeptide import PPBuilder

def diff_ang(pdb1, pdb2):   
    ppb=PPBuilder()
    par=PDBParser()
    
    struct1=par.get_structure('CONF1', pdb1)
    struct2=par.get_structure('CONF2', pdb2)
    
    nchains1=list(struct1.get_chains())
    if nchains1:
        if len(nchains1)!=1:
            raise ValueError  
    else:
        raise ValueError
 
    nchains2=list(struct2.get_chains())
    if nchains2:
        if len(nchains2)!=1:
            raise ValueError    
    else:
        raise ValueError
            
    pp1=ppb.build_peptides(struct1)    
    pp2=ppb.build_peptides(struct2)
    
    l1=pp1[0].get_phi_psi_list()
    l2=pp2[0].get_phi_psi_list()
    
    if len(l1)!=len(l2):
        raise Exception("The chains cannot be compared: different lengths!")
    
    res=[]
    for t1,t2 in zip(l1,l2):
        if t2[0] is None or t1[0] is None:
            delta_phi=None
        else:
            delta_phi=t2[0]-t1[0]
        if t2[1] is None:
            delta_psi=None
        else:
            delta_psi=t2[1]-t1[1]
        res.append((delta_phi, delta_psi))
    return res
    