# -*- coding: utf-8 -*-

#4a) In una sequenza proteica, lo stesso amminoacido può trovarsi in posizioni 
# consecutive a formare una sottosequenza con caratteri tutti uguali. 
# Qualora tali sottosequenze siano precedute e seguite da un ammonoacido di 
# tipo diverso, le chiameremo "isole".
# Scrivere una funzione spot_islands(pdbfile) che, preso in ingresso un file 
# PDB relativo a una struttura a catena singola, restituisca la lista delle 
# isole presenti nella sequenza relativa, ordinate in ordine di comparizione.
# Ogni isola sarà descritta da una tupla che contiene l'indice del primo 
# elemento dell'isola con convenzione Python, e la sua sequenza.
#
# Es: in x.pdb ho una struttura con sequenza "MTMMARKMKDTDSEEEIREA"
#     spot_islands("x.pdb") --> [(2,"MM"),(13,"EEE")]

from Bio.PDB.PDBParser import PDBParser
from Bio.PDB.Polypeptide import PPBuilder

def spot_islands(pdbfile):

    struct1=PDBParser().get_structure('CONF1', pdbfile)
    nchains1=list(struct1.get_chains())
    if nchains1:
        if len(nchains1)!=1:
            raise ValueError  
    else:
        raise ValueError
        
    ppeps=PPBuilder().build_peptides(struct1)
    aaseq=str(ppeps[0].get_sequence())
   
    islands=[]   
    prev_res=""
    curr_iseq=""
    curr_istart=-1
    for i,c in enumerate(aaseq):
        if c==prev_res:
            curr_iseq+=c
            if curr_istart+1==i:
                curr_iseq+=c
        else:
            if curr_iseq:
                islands.append((curr_istart,curr_iseq))
                curr_iseq=""
            curr_istart=i
            prev_res=c
    return islands
        